package ru.dbotthepony.kstarbound.defs.item.api

import com.google.gson.Gson
import com.google.gson.TypeAdapter
import com.google.gson.TypeAdapterFactory
import com.google.gson.reflect.TypeToken
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kommons.gson.consumeNull
import ru.dbotthepony.kstarbound.defs.image.SpriteReference
import ru.dbotthepony.kstarbound.json.builder.FactoryAdapter
import ru.dbotthepony.kstarbound.json.builder.JsonFactory
import ru.dbotthepony.kstarbound.json.builder.JsonImplementation

interface IArmorItemDefinition : ILeveledItemDefinition, IScriptableItemDefinition {
	/**
	 * Варианты покраски (???)
	 */
	val colorOptions: List<Map<String, String>>

	/**
	 * Визуальные кадры анимации, когда надето на гуманоида мужского пола
	 */
	val maleFrames: IFrames

	/**
	 * Визуальные кадры анимации, когда надето на гуманоида женского пола
	 */
	val femaleFrames: IFrames

	@JsonImplementation(Frames::class)
	interface IFrames {
		val body: SpriteReference
		val backSleeve: SpriteReference?
		val frontSleeve: SpriteReference?
	}

	data class Frames(
		override val body: SpriteReference,
		override val backSleeve: SpriteReference? = null,
		override val frontSleeve: SpriteReference? = null,
	) : IFrames {
		object Factory : TypeAdapterFactory {
			override fun <T : Any?> create(gson: Gson, type: TypeToken<T>): TypeAdapter<T>? {
				if (type.rawType == Frames::class.java) {
					return object : TypeAdapter<Frames>() {
						private val adapter = FactoryAdapter.createFor(Frames::class, gson)

						private val frames = gson.getAdapter(SpriteReference::class.java)

						override fun write(out: JsonWriter, value: Frames?) {
							adapter.write(out, value)
						}

						override fun read(`in`: JsonReader): Frames? {
							if (`in`.consumeNull())
								return null
							else if (`in`.peek() == JsonToken.STRING)
								return Frames(frames.read(`in`), null, null)
							else
								return adapter.read(`in`)
						}
					} as TypeAdapter<T>
				}

				return null
			}
		}
	}
}
